==> linuxconf-1.21.src.tar.gz: linuxconf-1.21/modules/gurus/gurupath.cc <==
#include <string.h>
#include "gurus.h"
#include "gurus.m"
#include <proto.h>

PUBLIC GURUPATH_STATUS::GURUPATH_STATUS()
{
	is_possible = false;
	is_filled = false;
	memset (tb,0,sizeof(tb));
	nbpath = 0;
	wearehere = false;
	some_errors = false;
	level = 0;
}

PUBLIC GURUPATH_STATUS::~GURUPATH_STATUS()
{
	for (unsigned i=0; i<sizeof(tb)/sizeof(tb[0]); i++){
		delete tb[i];
	}
}

int _F_gurupath::path1 (
	GURUPATH_MODE,
	GURUPATH_STATUS &)
{
	return -2;
}
int _F_gurupath::path2 (
	GURUPATH_MODE,
	GURUPATH_STATUS &)
{
	return -2;
}
int _F_gurupath::path3 (
	GURUPATH_MODE,
	GURUPATH_STATUS &)
{
	return -2;
}

static int gurupath_eval (_F_gurupath &c, GURUPATH_STATUS &status)
{
	int ret = c.eval (GURUPATH_EVAL,status);
	status.nbpath = 0;
	for (int i=0; i<MAXGURUPATH; i++){
		GURUPATH_STATUS *st = new GURUPATH_STATUS;
		st->level = status.level + 1;
		int tmp = -2;
		if (i==0){
			tmp = c.path1 (GURUPATH_EVAL,*st);
		}else if (i==1){
			tmp = c.path2 (GURUPATH_EVAL,*st);
		}else if (i==3){
			tmp = c.path3 (GURUPATH_EVAL,*st);
		}
		delete status.tb[i];
		status.tb[i] = NULL;
		if (tmp != -2){
			status.nbpath++;
			status.tb[i] = st;
		}else{
			delete st;
			break;
		}
	}
	return ret;
}

#if 0
static void status_dump (GURUPATH_STATUS &status, int level)
{
	printf ("%*spossible %d filled %d nb %d\n",level*4,"",status.is_possible
		,status.is_filled,status.nbpath);
	for (int i=0; i<status.nbpath; i++){
		status_dump (*status.tb[i],level+1);
	}
}
#endif

struct gurupath_private {
	DIALOG *dia;
	SSTRING title;
	SSTRING intro;
	int nof;
	int butmask;
	MENU_STATUS code;
	GURUPATH_STATUS *status;
	SSTRING terminal;
};


/*
	Evaluate the dimension of the map.
	Return the height.
*/
static int gurupath_evaldim (GURUPATH_STATUS &status, int &maxlevel)
{
	int ret = 1;
	if (status.level > maxlevel) maxlevel = status.level;
	if (status.nbpath > 0){
		ret = 0;
		for (int i=0; i<status.nbpath; i++){
			ret += gurupath_evaldim (*status.tb[i],maxlevel);
		}
	}
	return ret;
}

static char guictx[20];
struct GURUPATH_DRAWINFO{
	GURUPATH_STATUS *status;
	char ctx[20];
	int y;
	int x;
};

static void gurupath_draw (
	GURUPATH_STATUS &status,
	int lastx,
	int lasty,
	int newx,
	int newy)
{
	static bool is_init = false;
	if (!is_init){
		is_init = true;
		diagui_sendcmd (P_Defpen,"penblue blue 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushblue blue 0\n");
		diagui_sendcmd (P_Defdc,"dcblue $pen=penblue brush=brushblue\n");
		diagui_sendcmd (P_Defpen,"penred red 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushred red 0\n");
		diagui_sendcmd (P_Defdc,"dcred $pen=penred brush=brushred\n");
		diagui_sendcmd (P_Defpen,"penblack black 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushblack black 0\n");
		diagui_sendcmd (P_Defdc,"dcblack $pen=penblack brush=brushblack\n");
		diagui_sendcmd (P_Defpen,"penyellow yellow 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushyellow yellow 0\n");
		diagui_sendcmd (P_Defdc,"dcyellow $pen=penyellow brush=brushyellow\n");
		diagui_sendcmd (P_Defpen,"gurupenwhite white 1 0\n");
		diagui_sendcmd (P_Defbrush,"gurubrushwhite white 0\n");
		diagui_sendcmd (P_Defdc,"dcguruwhite $pen=gurupenwhite brush=gurubrushwhite\n");
	}
	const char *dc = "dcblack";
	if (status.is_possible){
		if (status.is_filled){
			dc = "dcblue";
		}
	}else{
		dc = "dcred";
	}
	diagui_sendcmd (P_Drawline,"%d %d %d %d $dc=%s\n",lastx,lasty,newx,newy
		,dc);
	if (status.nbpath > 0){
		if (status.nbpath == 1){
			gurupath_draw (*status.tb[0],newx,newy,newx+30,newy);
		}else{
			int tbh[status.nbpath],total=0;
			for (int i=0; i<status.nbpath; i++){
				int dummy=0;
				int height = gurupath_evaldim (*status.tb[i],dummy);
				total += height;
				tbh[i] = height;
			}
			// int h = (status.nbpath-1)*30/2;
			int h = (total-1)*30/2;
			int nexty = newy-h;
			int nextx = newx+30;
			for (int i=0; i<status.nbpath; i++){
				gurupath_draw (*status.tb[i],newx,newy,nextx,nexty);
				nexty += tbh[i]*30;
			}
		}
	}else if (!status.title.is_empty()){
		// Ok, this is a terminal node
		char tmp[1000];
		diagui_sendcmd (P_Drawtext,"%d %d %s\n",newx+10,newy
			,diagui_quote (status.title.get(),tmp));

	}
	if (status.some_errors && !status.wearehere){
		// Ok, a node with an invalid state. The user must fix it
		// We draw a broken circle (two half circle teared apart)
		dc = "dcred";
		int x0 = newx - 6;
		int y0 = newy - 6;
		diagui_sendcmd (P_Fillarc,"%d %d %d %d %d %d $dc=%s\n"
			,x0,y0,10,10,64*45,64*180,dc);
		x0 = newx - 4;
		y0 = newy - 4;
		diagui_sendcmd (P_Fillarc,"%d %d %d %d %d %d $dc=%s\n"
			,x0,y0,10,10,64*225,64*180,dc);
	}else{
		// Now we put the circle in place
		dc = "dcblack";
		if (status.wearehere){
			dc = "dcyellow";
		}
		int x0 = newx - 5;
		int y0 = newy - 5;
		diagui_sendcmd (P_Fillarc,"%d %d %d %d %d %d $dc=%s\n",x0,y0,10,10,0,64*360,dc);
		if (!status.wearehere){
			// Draw the center of the circle
			if (status.is_filled){
				dc = "dcblue";
			}else{
				dc = "dcguruwhite";
			}
			int x0 = newx - 3;
			int y0 = newy - 3;
			diagui_sendcmd (P_Fillarc,"%d %d %d %d %d %d $dc=%s\n",x0,y0,6,6,0,64*360,dc);
		}
	}
}

static GURUPATH_DRAWINFO *dinfo;

static void gurupath_draw ()
{
	diagui_sendcmd (P_Setcontext,"%s\n",dinfo->ctx);
	diagui_sendcmd (P_Clear,"\n");
	gurupath_draw (*dinfo->status,dinfo->x,dinfo->y,dinfo->x+30,dinfo->y);
	diagui_sendcmd (P_Refresh,"\n");
	diagui_sendcmd (P_End,"\n");
}

/*
	Format a long text into many labels.
*/
static void gurupath_formatintro (DIALOG &dia, const char *text)
{
	char line[strlen(text)+1];
	while (*text != '\0'){
		char *dst = line;
		while (*text != '\n' && *text != '\0') *dst++ = *text++;
		*dst = '\0';
		if (*text == '\n') text++;
		dia.gui_label (line);
		dia.gui_dispolast (GUI_H_LEFT,2,GUI_V_CENTER,1);
		dia.newline();
	}
}

void _F_gurupath::setintro (const char *text)
{
	priv->intro.setfrom (text);
	gurupath_formatintro (*priv->dia,text);
}

void _F_gurupath::settitle (const char *title)
{
	static bool is_sent = false;
	if (!is_sent){
		is_sent = true;
		diagui_sendcmd (P_Deffont,"fontgurutitle 24 2 1 1 0\n");
		diagui_sendcmd (P_Defpen,"pengurutitle blue 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushgurutitle blue 0\n");
		diagui_sendcmd (P_Defdc,"dcgurutitle $font=fontgurutitle pen=pengurutitle brush=brushgurutitle\n");
	}
	char tmp[1000];
	priv->dia->gui_passthrough (P_Label,"%s $dc=dcgurutitle",diagui_quote(title,tmp));
	priv->dia->gui_dispolast (GUI_H_CENTER,2,GUI_V_CENTER,1);
	priv->dia->newline();
	priv->title.setfrom (title);
}

bool _F_gurupath::edit (HELP_FILE &help)
{
	if (dialog_mode == DIALOG_GUI){
		priv->code = priv->dia->edit ("","",help,priv->nof
			,priv->butmask);
	}else{
		priv->code = priv->dia->edit (priv->title.get(),priv->intro.get(),help,priv->nof
			,priv->butmask);
	}
	bool ret = priv->code == MENU_USR2;
	if (ret) priv->dia->save();
	return ret;
}

bool _F_gurupath::edit ()
{
	return edit (help_nil);
}

/*
	Record the information about a terminal node
*/
void _F_gurupath::setterminal (const char *shorttext, const char *longtext)
{
	priv->status->title.setfrom (shorttext);
	priv->terminal.setfrom (longtext);
}

int gurupath(
	_F_gurupath &c,
	GURUPATH_MODE mode,
	GURUPATH_STATUS &status)
{
	gurupath_private priv;
	c.priv = &priv;
	priv.nof = 0;
	priv.dia = NULL;
	priv.status = &status;
	int ret = -1;
	if (mode == GURUPATH_EVAL){
		ret = gurupath_eval (c,status);
	}else{
		while (1){
			status.wearehere = true;
			gurupath_draw ();
			status.wearehere = false;
			{
				DIALOG dia;
				dia.setcontext (guictx);
				priv.butmask = MENUBUT_USR2;
				if (status.level > 0){
					dia.setbutinfo (MENU_USR1,MSG_U(B_BACK,"Back"),MSG_R(B_BACK));
					priv.butmask |= MENUBUT_USR1;
				}else{
					priv.butmask |= MENUBUT_CANCEL;
				}
				dia.setbutinfo (MENU_USR2,MSG_U(B_NEXT,"Next"),MSG_R(B_NEXT));
				priv.dia = &dia;
				c.dialog (dia);
				priv.dia = NULL;
			}
			if (priv.code == MENU_CANCEL || priv.code == MENU_ESCAPE){
				ret = -1;
				break;
			}else if (priv.code == MENU_USR1){
				ret = 1;
				break;
			}else if (priv.code == MENU_USR2){
				ret = 0;
				gurupath_eval (c,status);
				// status_dump(status,0);
				if (status.nbpath > 0){
					for (int i=0; ret != -1 && ret != 1 && i<status.nbpath; i++){
						GURUPATH_STATUS *st = status.tb[i];
						if (st->is_possible){
							if (i==0){
								ret = c.path1 (mode,*st);
							}else if (i==1){
								ret = c.path2 (mode,*st);
							}else if (i==2){
								ret = c.path3 (mode,*st);
							}
						}
					}
					if (ret != 1) break;
				}else{
					if (!priv.terminal.is_empty()){
					}
					break;
				}
			}
		}
	}
	return ret;
}

/*
	Return -1 if the user aborted the process
*/
int gurumanage(_F_gurumanage &c,
	const char *title,
	const char *intro)
{
	GURUPATH_STATUS status;
	c.exec (GURUPATH_EVAL,status);

	// status_dump (status,0);
	int nbcol=0;
	int nbline = gurupath_evaldim (status,nbcol);
	int height = (nbline-1)*30+40;
	char mainid[10];
	extern int uithread_id;
	sprintf (mainid,"guru-%d",uithread_id);
	sprintf (guictx,"%s.dialog",mainid);
	char tmp[1000];
	dialog_clear();
	diagui_sendcmd (P_MainForm,"%s %s\n",mainid,diagui_quote(title,tmp));
	diagui_sendcmd (P_Form,"map $w=400 h=%d\n",height);
	diagui_sendcmd (P_End,"\n");
	diagui_sendcmd (P_Newline,"\n");
	diagui_sendcmd (P_Form,"dialog\n");
	diagui_sendcmd (P_End,"\n");
	diagui_sendcmd (P_End,"\n");
	//diagui_sendcmd (P_End,"$nopopup=1\n");

	GURUPATH_DRAWINFO drawinfo;
	sprintf (drawinfo.ctx,"%s.map",mainid);
	drawinfo.status = &status;
	drawinfo.x = 30;
	drawinfo.y = height/2;
	dinfo = &drawinfo;
	gurupath_draw ();

	MENU_STATUS code;
	{
		DIALOG dia;
		dia.setcontext (guictx);
		const char *text_intro = "";
		if (dialog_mode == DIALOG_GUI){
			gurupath_formatintro (dia,MSG_U(I_GURUINTRO
				,"You will be presented various dialogs. They are mapped\n"
				 "along the map above. Depending on your answers,\n"
				 "you will visit apropriate areas of the map.\n"
				 "\n"
				 "The drawing above (the map) represent the path to follow.\n"
				 "    -The yellow X represent your position.\n"
				 "    -Red lines indicate non reachable areas.\n"
				 "    -blue lines indicate already filled dialogs.\n"
				 "    -black lines represent area to visit.\n"
				 ));
			dia.gui_label (""); dia.newline();
			dia.gui_label (""); dia.newline();
			gurupath_formatintro (dia,intro);
		}else{
			text_intro = intro;
		}
		dia.setbutinfo (MENU_USR1,MSG_R(B_NEXT),MSG_R(B_NEXT));
		int nof = 0;
		code = dia.edit ("",text_intro,help_nil,nof,MENUBUT_CANCEL|MENUBUT_USR1);
	}
	int ret = -1;
	if (code == MENU_USR1){
		ret = c.exec (GURUPATH_EDIT,status);
	}
	diagui_sendcmd (P_Delete,"%s\n",mainid);
	return ret;
}

/*
	Use this function at the end of a branch
*/
int gurupath_confirm (
	GURUPATH_MODE mode,
	GURUPATH_STATUS &status,
	const char *branch_title,	// The short title at the end
								// of a branch on the map
	const char *text)			// Some longer confirmation)
{
	status.is_possible = true;
	status.is_filled = false;
	status.some_errors = false;
	status.title.setfrom (branch_title);
	if (mode == GURUPATH_EDIT){
		xconf_notice ("%s",text);
	}
	return 0;
}

==> linuxconf-1.22.src.tar.gz: linuxconf-1.22/modules/gurus/gurupath.cc <==
#include <string.h>
#include "gurus.h"
#include "gurus.m"
#include <proto.h>

static HELP_FILE help_gurus ("gurus","gurus");

PUBLIC GURUPATH_STATUS::GURUPATH_STATUS()
{
	is_possible = false;
	is_filled = false;
	memset (tb,0,sizeof(tb));
	nbpath = 0;
	wearehere = false;
	some_errors = false;
	level = 0;
}

PUBLIC GURUPATH_STATUS::~GURUPATH_STATUS()
{
	for (unsigned i=0; i<sizeof(tb)/sizeof(tb[0]); i++){
		delete tb[i];
	}
}

int _F_gurupath::path1 (
	GURUPATH_MODE,
	GURUPATH_STATUS &)
{
	return -2;
}
int _F_gurupath::path2 (
	GURUPATH_MODE,
	GURUPATH_STATUS &)
{
	return -2;
}
int _F_gurupath::path3 (
	GURUPATH_MODE,
	GURUPATH_STATUS &)
{
	return -2;
}

static int gurupath_eval (_F_gurupath &c, GURUPATH_STATUS &status)
{
	int ret = c.eval (GURUPATH_EVAL,status);
	status.nbpath = 0;
	for (int i=0; i<MAXGURUPATH; i++){
		GURUPATH_STATUS *st = new GURUPATH_STATUS;
		st->level = status.level + 1;
		int tmp = -2;
		if (i==0){
			tmp = c.path1 (GURUPATH_EVAL,*st);
		}else if (i==1){
			tmp = c.path2 (GURUPATH_EVAL,*st);
		}else if (i==3){
			tmp = c.path3 (GURUPATH_EVAL,*st);
		}
		delete status.tb[i];
		status.tb[i] = NULL;
		if (tmp != -2){
			status.nbpath++;
			status.tb[i] = st;
		}else{
			delete st;
			break;
		}
	}
	return ret;
}

#if 0
static void status_dump (GURUPATH_STATUS &status, int level)
{
	printf ("%*spossible %d filled %d nb %d\n",level*4,"",status.is_possible
		,status.is_filled,status.nbpath);
	for (int i=0; i<status.nbpath; i++){
		status_dump (*status.tb[i],level+1);
	}
}
#endif

struct gurupath_private {
	DIALOG *dia;
	SSTRING title;
	SSTRING intro;
	int nof;
	int butmask;
	MENU_STATUS code;
	GURUPATH_STATUS *status;
	SSTRING terminal;
	int tbid[10];	// Id for extra buttons of the dialogs
	int nbid;
};


/*
	Evaluate the dimension of the map.
	Return the height.
*/
static int gurupath_evaldim (GURUPATH_STATUS &status, int &maxlevel)
{
	int ret = 1;
	if (status.level > maxlevel) maxlevel = status.level;
	if (status.nbpath > 0){
		ret = 0;
		for (int i=0; i<status.nbpath; i++){
			ret += gurupath_evaldim (*status.tb[i],maxlevel);
		}
	}
	return ret;
}

static char guictx[20];
struct GURUPATH_DRAWINFO{
	GURUPATH_STATUS *status;
	char ctx[20];
	int y;
	int x;
};

static void gurupath_draw (
	GURUPATH_STATUS &status,
	int lastx,
	int lasty,
	int newx,
	int newy)
{
	static bool is_init = false;
	if (!is_init){
		is_init = true;
		diagui_sendcmd (P_Defpen,"penblue blue 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushblue blue 0\n");
		diagui_sendcmd (P_Defdc,"dcblue $pen=penblue brush=brushblue\n");
		diagui_sendcmd (P_Defpen,"penred red 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushred red 0\n");
		diagui_sendcmd (P_Defdc,"dcred $pen=penred brush=brushred\n");
		diagui_sendcmd (P_Defpen,"penblack black 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushblack black 0\n");
		diagui_sendcmd (P_Defdc,"dcblack $pen=penblack brush=brushblack\n");
		diagui_sendcmd (P_Defpen,"penyellow yellow 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushyellow yellow 0\n");
		diagui_sendcmd (P_Defdc,"dcyellow $pen=penyellow brush=brushyellow\n");
		diagui_sendcmd (P_Defpen,"gurupenwhite white 1 0\n");
		diagui_sendcmd (P_Defbrush,"gurubrushwhite white 0\n");
		diagui_sendcmd (P_Defdc,"dcguruwhite $pen=gurupenwhite brush=gurubrushwhite\n");
	}
	const char *dc = "dcblack";
	if (status.is_possible){
		if (status.is_filled){
			dc = "dcblue";
		}
	}else{
		dc = "dcred";
	}
	diagui_sendcmd (P_Drawline,"%d %d %d %d $dc=%s\n",lastx,lasty,newx,newy
		,dc);
	if (status.nbpath > 0){
		if (status.nbpath == 1){
			gurupath_draw (*status.tb[0],newx,newy,newx+30,newy);
		}else{
			int tbh[status.nbpath],total=0;
			for (int i=0; i<status.nbpath; i++){
				int dummy=0;
				int height = gurupath_evaldim (*status.tb[i],dummy);
				total += height;
				tbh[i] = height;
			}
			// int h = (status.nbpath-1)*30/2;
			int h = (total-1)*30/2;
			int nexty = newy-h;
			int nextx = newx+30;
			for (int i=0; i<status.nbpath; i++){
				gurupath_draw (*status.tb[i],newx,newy,nextx,nexty);
				nexty += tbh[i]*30;
			}
		}
	}else if (!status.title.is_empty()){
		// Ok, this is a terminal node
		char tmp[1000];
		diagui_sendcmd (P_Drawtext,"%d %d %s\n",newx+10,newy
			,diagui_quote (status.title.get(),tmp));

	}
	if (status.some_errors && !status.wearehere){
		// Ok, a node with an invalid state. The user must fix it
		// We draw a broken circle (two half circle teared apart)
		dc = "dcred";
		int x0 = newx - 6;
		int y0 = newy - 6;
		diagui_sendcmd (P_Fillarc,"%d %d %d %d %d %d $dc=%s\n"
			,x0,y0,10,10,64*45,64*180,dc);
		x0 = newx - 4;
		y0 = newy - 4;
		diagui_sendcmd (P_Fillarc,"%d %d %d %d %d %d $dc=%s\n"
			,x0,y0,10,10,64*225,64*180,dc);
	}else{
		// Now we put the circle in place
		dc = "dcblack";
		if (status.wearehere){
			dc = "dcyellow";
		}
		int x0 = newx - 5;
		int y0 = newy - 5;
		diagui_sendcmd (P_Fillarc,"%d %d %d %d %d %d $dc=%s\n",x0,y0,10,10,0,64*360,dc);
		if (!status.wearehere){
			// Draw the center of the circle
			if (status.is_filled){
				dc = "dcblue";
			}else{
				dc = "dcguruwhite";
			}
			int x0 = newx - 3;
			int y0 = newy - 3;
			diagui_sendcmd (P_Fillarc,"%d %d %d %d %d %d $dc=%s\n",x0,y0,6,6,0,64*360,dc);
		}
	}
}

static GURUPATH_DRAWINFO *dinfo;
static HELP_FILE *guruhelp;
static const char *gurutitle;

static void gurupath_draw ()
{
	diagui_sendcmd (P_Setcontext,"%s\n",dinfo->ctx);
	diagui_sendcmd (P_Clear,"\n");
	gurupath_draw (*dinfo->status,dinfo->x,dinfo->y,dinfo->x+30,dinfo->y);
	diagui_sendcmd (P_Refresh,"\n");
	diagui_sendcmd (P_End,"\n");
}

/*
	Format a long text into many labels.
*/
static void gurupath_formatintro (DIALOG &dia, const char *text)
{
	char line[strlen(text)+1];
	while (*text != '\0'){
		char *dst = line;
		while (*text != '\n' && *text != '\0') *dst++ = *text++;
		*dst = '\0';
		if (*text == '\n') text++;
		dia.gui_label (line);
		dia.gui_dispolast (GUI_H_LEFT,2,GUI_V_CENTER,1);
		dia.newline();
	}
}

void _F_gurupath::setintro (const char *text)
{
	priv->intro.setfrom (text);
	gurupath_formatintro (*priv->dia,text);
}

void _F_gurupath::settitle (const char *title)
{
	static bool is_sent = false;
	if (!is_sent){
		is_sent = true;
		diagui_sendcmd (P_Deffont,"fontgurutitle 24 2 1 1 0\n");
		diagui_sendcmd (P_Defpen,"pengurutitle blue 1 0\n");
		diagui_sendcmd (P_Defbrush,"brushgurutitle blue 0\n");
		diagui_sendcmd (P_Defdc,"dcgurutitle $font=fontgurutitle pen=pengurutitle brush=brushgurutitle\n");
	}
	char tmp[1000];
	priv->dia->gui_passthrough (P_Label,"%s $dc=dcgurutitle",diagui_quote(title,tmp));
	priv->dia->gui_dispolast (GUI_H_CENTER,2,GUI_V_CENTER,1);
	priv->dia->newline();
	priv->title.setfrom (title);
}

bool _F_gurupath::edit (HELP_FILE &help)
{
	while (1){
		if (dialog_mode == DIALOG_GUI){
			priv->code = priv->dia->edit ("","",help,priv->nof
				,priv->butmask);
		}else{
			priv->code = priv->dia->edit (priv->title.get(),priv->intro.get(),help,priv->nof
				,priv->butmask);
		}
		if (priv->code >= MENU_USR3){
			dobutton (priv->tbid[priv->code - MENU_USR3]);
			priv->dia->reload();
		}else{
			break;
		}
	}
	bool ret = priv->code == MENU_USR2;
	if (ret) priv->dia->save();
	return ret;
}

bool _F_gurupath::edit ()
{
	return edit (help_nil);
}

void _F_gurupath::dobutton(int)
{
}

void _F_gurupath::setbutinfo (int id, const char *s1, const char *s2)
{
	priv->dia->setbutinfo (MENU_USR3+priv->nbid,s1,s2);
	priv->tbid[priv->nbid] = id;
	priv->butmask |= (MENUBUT_USR3 << priv->nbid);
	priv->nbid++;
}

/*
	Record the information about a terminal node
*/
void _F_gurupath::setterminal (const char *shorttext, const char *longtext)
{
	priv->status->title.setfrom (shorttext);
	priv->terminal.setfrom (longtext);
}

int gurupath(
	_F_gurupath &c,
	GURUPATH_MODE mode,
	GURUPATH_STATUS &status)
{
	gurupath_private priv;
	c.priv = &priv;
	priv.nof = 0;
	priv.dia = NULL;
	priv.status = &status;
	priv.nbid = 0;
	int ret = -1;
	if (mode == GURUPATH_EVAL){
		ret = gurupath_eval (c,status);
	}else{
		while (1){
			status.wearehere = true;
			gurupath_draw ();
			status.wearehere = false;
			{
				DIALOG dia;
				if (guruhelp != NULL){
					dia.addhelp (*guruhelp,gurutitle);
				}
				dia.addhelp (help_gurus,MSG_U(I_GURUSINTRO,"Gurus general introduction"));
				dia.setcontext (guictx);
				priv.butmask = MENUBUT_USR2;
				if (status.level > 0){
					dia.setbutinfo (MENU_USR1,MSG_U(B_BACK,"Back"),MSG_R(B_BACK));
					priv.butmask |= MENUBUT_USR1;
				}else{
					priv.butmask |= MENUBUT_CANCEL;
				}
				dia.setbutinfo (MENU_USR2,MSG_U(B_NEXT,"Next"),MSG_R(B_NEXT));
				priv.dia = &dia;
				c.dialog (dia);
				priv.dia = NULL;
			}
			if (priv.code == MENU_CANCEL || priv.code == MENU_ESCAPE){
				ret = -1;
				break;
			}else if (priv.code == MENU_USR1){
				ret = 1;
				break;
			}else if (priv.code == MENU_USR2){
				ret = 0;
				gurupath_eval (c,status);
				// status_dump(status,0);
				if (status.nbpath > 0){
					for (int i=0; ret != -1 && ret != 1 && i<status.nbpath; i++){
						GURUPATH_STATUS *st = status.tb[i];
						if (st->is_possible){
							if (i==0){
								ret = c.path1 (mode,*st);
							}else if (i==1){
								ret = c.path2 (mode,*st);
							}else if (i==2){
								ret = c.path3 (mode,*st);
							}
						}
					}
					if (ret != 1) break;
				}else{
					if (!priv.terminal.is_empty()){
					}
					break;
				}
			}
		}
	}
	return ret;
}

/*
	Return -1 if the user aborted the process
*/
int gurumanage(_F_gurumanage &c,
	const char *title,
	const char *intro,
	HELP_FILE &help)
{
	GURUPATH_STATUS status;
	c.exec (GURUPATH_EVAL,status);

	// status_dump (status,0);
	int nbcol=0;
	int nbline = gurupath_evaldim (status,nbcol);
	int height = (nbline-1)*30+40;
	char mainid[10];
	extern int uithread_id;
	sprintf (mainid,"guru-%d",uithread_id);
	sprintf (guictx,"%s.dialog",mainid);
	char tmp[1000];
	dialog_clear();
	diagui_sendcmd (P_MainForm,"%s %s\n",mainid,diagui_quote(title,tmp));
	diagui_sendcmd (P_Form,"map $w=400 h=%d\n",height);
	diagui_sendcmd (P_End,"\n");
	diagui_sendcmd (P_Newline,"\n");
	diagui_sendcmd (P_Form,"dialog\n");
	diagui_sendcmd (P_End,"\n");
	diagui_sendcmd (P_End,"$nopopup=1\n");

	GURUPATH_DRAWINFO drawinfo;
	sprintf (drawinfo.ctx,"%s.map",mainid);
	drawinfo.status = &status;
	drawinfo.x = 30;
	drawinfo.y = height/2;
	dinfo = &drawinfo;
	gurupath_draw ();

	MENU_STATUS code;
	{
		DIALOG dia;
		dia.setcontext (guictx);
		const char *text_intro = "";
		if (dialog_mode == DIALOG_GUI){
			gurupath_formatintro (dia,MSG_U(I_GURUINTRO
				,"You will be presented various dialogs. They are mapped\n"
				 "along the map above. Depending on your answers,\n"
				 "you will visit apropriate areas of the map.\n"
				 "\n"
				 "The drawing above (the map) represent the path to follow.\n"
				 "    -The yellow X represent your position.\n"
				 "    -Red lines indicate non reachable areas.\n"
				 "    -blue lines indicate already filled dialogs.\n"
				 "    -black lines represent area to visit.\n"
				 ));
			dia.gui_label (""); dia.newline();
			dia.gui_label (""); dia.newline();
			gurupath_formatintro (dia,intro);
		}else{
			text_intro = intro;
		}
		dia.setbutinfo (MENU_USR1,MSG_R(B_NEXT),MSG_R(B_NEXT));
		int nof = 0;
		dia.addhelp (help_gurus,MSG_R(I_GURUSINTRO));
		code = dia.edit ("",text_intro,help,nof,MENUBUT_CANCEL|MENUBUT_USR1);
	}
	int ret = -1;
	if (code == MENU_USR1){
		guruhelp = NULL;
		gurutitle = title;
		if (!help.is_nil()) guruhelp = &help;
		ret = c.exec (GURUPATH_EDIT,status);
	}
	diagui_sendcmd (P_Delete,"%s\n",mainid);
	return ret;
}

/*
	Use this function at the end of a branch
*/
int gurupath_confirm (
	GURUPATH_MODE mode,
	GURUPATH_STATUS &status,
	const char *branch_title,	// The short title at the end
								// of a branch on the map
	const char *text)			// Some longer confirmation)
{
	status.is_possible = true;
	status.is_filled = false;
	status.some_errors = false;
	status.title.setfrom (branch_title);
	if (mode == GURUPATH_EDIT){
		xconf_notice ("%s",text);
	}
	return 0;
}
